@@ -12,6 +12,9 @@ from orangecontrib.xoppy.util.xoppy_xraylib_util import xpower_calc
 from oasys.widgets.exchange import DataExchangeObject
 from orangecontrib.xoppy.widgets.gui.ow_xoppy_widget import XoppyWidget
 
+
+import scipy.constants as codata
+
 class OWxpower(XoppyWidget):
     name = "POWER"
     id = "orange.widgets.dataxpower"
@@ -19,11 +22,11 @@ class OWxpower(XoppyWidget):
     icon = "icons/xoppy_xpower.png"
     priority = 3
     category = ""
-    keywords = ["xoppy", "xpower"]
+    keywords = ["xoppy", "power"]
 
     inputs = [("ExchangeData", DataExchangeObject, "acceptExchangeData")]
 
-    SOURCE = Setting(0)
+    SOURCE = Setting(2)
     ENER_MIN = Setting(1000.0)
     ENER_MAX = Setting(50000.0)
     ENER_N = Setting(100)
@@ -75,9 +78,9 @@ class OWxpower(XoppyWidget):
         #widget index 2 
         idx += 1 
         box1 = gui.widgetBox(box) 
-        gui.comboBox(box1, self, "SOURCE",
+        self.box_source = gui.comboBox(box1, self, "SOURCE",
                      label=self.unitLabels()[idx], addSpace=False,
-                    items=['Normalized to 1 (Standard E grid)  ', 'Normalized to 1 (E from keyboard)  ', 'From external file.                '],
+                    items=['From Oasys wire', 'Normalized to 1', 'From external file.                '],
                     valueType=int, orientation="horizontal", labelWidth=150)
         self.show_at(self.unitFlags()[idx], box1)
         
@@ -391,6 +394,9 @@ class OWxpower(XoppyWidget):
                     valueType=int, orientation="horizontal", labelWidth=250)
         self.show_at(self.unitFlags()[idx], box1)
 
+        self.input_spectrum = None
+
+
     def set_NELEMENTS(self):
         self.initializeTabs()
 
@@ -398,11 +404,11 @@ class OWxpower(XoppyWidget):
         self.initializeTabs()
 
     def unitLabels(self):
-         return ['Source:',
+         return ['Input beam:',
                  'From energy [eV]:      ',
                  'To energy [eV]:',
                  'Energy points:  ',
-                 'File with source:',
+                 'File with input beam spectral power:',
                  'Number of elements:',
                  '1st oe formula','kind:','Filter thick[mm]','Mirror angle[mrad]','Roughness[A]','Density [g/cm^3]',
                  '2nd oe formula','kind:','Filter thick[mm]','Mirror angle[mrad]','Roughness[A]','Density [g/cm^3]',
@@ -433,57 +439,86 @@ class OWxpower(XoppyWidget):
         self.le_source_file.setText(oasysgui.selectFileFromDialog(self, self.SOURCE_FILE, "Open Source File", file_extension_filter="*.*"))
 
     def acceptExchangeData(self, exchangeData):
+
+        self.input_spectrum = None
+        self.SOURCE = 0
+        # self.box_source.setCurrentIndex(self.SOURCE)
+
         try:
             if not exchangeData is None:
                 if exchangeData.get_program_name() == "XOPPY":
                     no_bandwidth = False
                     if exchangeData.get_widget_name() =="UNDULATOR_FLUX" :
-                        self.SOURCE_FILE = "xoppy_undulator_flux"
+                        # self.SOURCE_FILE = "xoppy_undulator_flux"
                         no_bandwidth = True
                         index_flux = 2
                     elif exchangeData.get_widget_name() == "BM" :
                         if exchangeData.get_content("is_log_plot") == 1:
                             raise Exception("Logaritmic X scale of Xoppy Energy distribution not supported")
                         if exchangeData.get_content("calculation_type") == 0 and exchangeData.get_content("psi") == 0:
-                            self.SOURCE_FILE = "xoppy_bm_flux"
+                            # self.SOURCE_FILE = "xoppy_bm_flux"
                             no_bandwidth = True
                             index_flux = 6
                         else:
                             raise Exception("Xoppy result is not an Flux vs Energy distribution integrated in Psi")
                     elif exchangeData.get_widget_name() =="XWIGGLER" :
-                        self.SOURCE_FILE = "xoppy_xwiggler_flux"
+                        # self.SOURCE_FILE = "xoppy_xwiggler_flux"
+                        no_bandwidth = True
+                        index_flux = 2
+                    elif exchangeData.get_widget_name() =="WS" :
+                        # self.SOURCE_FILE = "xoppy_xwiggler_flux"
                         no_bandwidth = True
                         index_flux = 2
                     elif exchangeData.get_widget_name() =="XTUBES" :
-                        self.SOURCE_FILE = "xoppy_xtubes_flux"
+                        # self.SOURCE_FILE = "xoppy_xtubes_flux"
                         index_flux = 1
                         no_bandwidth = True
                     elif exchangeData.get_widget_name() =="XTUBE_W" :
-                        self.SOURCE_FILE = "xoppy_xtube_w_flux"
+                        # self.SOURCE_FILE = "xoppy_xtube_w_flux"
                         index_flux = 1
                         no_bandwidth = True
                     elif exchangeData.get_widget_name() =="BLACK_BODY" :
-                        self.SOURCE_FILE = "xoppy_black_body_flux"
+                        # self.SOURCE_FILE = "xoppy_black_body_flux"
                         no_bandwidth = True
                         index_flux = 2
+
+                    elif exchangeData.get_widget_name() =="UNDULATOR_RADIATION" :
+                        # self.SOURCE_FILE = "xoppy_undulator_radiation"
+                        no_bandwidth = True
+                        index_flux = 1
+                    elif exchangeData.get_widget_name() =="POWER" :
+                        # self.SOURCE_FILE = "xoppy_undulator_power"
+                        no_bandwidth = True
+                        index_flux = -1
+                    elif exchangeData.get_widget_name() =="POWER3D" :
+                        # self.SOURCE_FILE = "xoppy_power3d"
+                        no_bandwidth = True
+                        index_flux = 1
+
                     else:
                         raise Exception("Xoppy Source not recognized")
 
-                    self.SOURCE_FILE += "_" + str(id(self)) + ".dat"
-                    self.SOURCE = 2
+                    # self.SOURCE_FILE += "_" + str(id(self)) + ".dat"
+
 
                     spectrum = exchangeData.get_content("xoppy_data")
-                    print(">>>>>>>>>>>>>>>",spectrum.shape)
-                    if not no_bandwidth:
-                        spectrum[:,index_flux] /= 0.001*spectrum[:,0]
 
-                    file = open(self.SOURCE_FILE, "w")
-                    for index in range(0, spectrum.shape[0]):
-                        file.write(str(spectrum[index, 0]) + " " + str(spectrum[index, index_flux]) + "\n")
+                    if exchangeData.get_widget_name() =="UNDULATOR_RADIATION" or \
+                        exchangeData.get_widget_name() =="POWER3D":
+                        [p, e, h, v ] = spectrum
+                        tmp = p.sum(axis=2).sum(axis=1)*(h[1]-h[0])*(v[1]-v[0])*codata.e*1e3
+                        spectrum = numpy.vstack((e,p.sum(axis=2).sum(axis=1)*(h[1]-h[0])*(v[1]-v[0])*
+                                                 codata.e*1e3))
+                        self.input_spectrum = spectrum
+                    else:
 
-                    file.close()
+                        if not no_bandwidth:
+                            spectrum[:,index_flux] /= 0.001*spectrum[:,0]
+
+                        self.input_spectrum = numpy.vstack((spectrum[:,0],spectrum[:,index_flux]))
 
                     self.process_showers()
+                    self.compute()
 
         except Exception as exception:
             QMessageBox.critical(self, "Error",
@@ -493,6 +528,8 @@ class OWxpower(XoppyWidget):
             #raise exception
 
 
+
+
     def check_fields(self):
 
         if self.SOURCE == 1:
@@ -570,8 +607,9 @@ class OWxpower(XoppyWidget):
     def extract_data_from_xoppy_output(self, calculation_output):
         return calculation_output
 
+
     def get_data_exchange_widget_name(self):
-        return "XPOWER"
+        return "POWER"
 
     def getKind(self, oe_n):
         if oe_n == 1:
@@ -601,7 +639,7 @@ class OWxpower(XoppyWidget):
     def getTitles(self):
         titles = []
 
-        if self.do_plot_intensity(): titles.append("Source")
+        if self.do_plot_intensity(): titles.append("Input beam")
 
         for oe_n in range(1, self.NELEMENTS+2):
             kind = self.getKind(oe_n)
@@ -617,7 +655,7 @@ class OWxpower(XoppyWidget):
                 if self.do_plot_local(): titles.append("[oe " + str(oe_n) + "] Im[n]=beta")
                 if self.do_plot_local(): titles.append("[oe " + str(oe_n) + "] delta/beta")
                 if self.do_plot_local(): titles.append("[oe " + str(oe_n) + "] Reflectivity-s")
-                if self.do_plot_local(): titles.append("[oe " + str(oe_n) + "] Transmitivity")
+                if self.do_plot_local(): titles.append("[oe " + str(oe_n) + "] Absorption")
                 if self.do_plot_intensity(): titles.append("Intensity after oe " + str(oe_n))
 
         return titles
@@ -755,24 +793,30 @@ class OWxpower(XoppyWidget):
 
 
         if self.SOURCE == 0:
-            energies = numpy.arange(0,500)
-            elefactor = numpy.log10(10000.0 / 30.0) / 300.0
-            energies = 10.0 * 10**(energies * elefactor)
-            source = numpy.ones(energies.size)
-            tmp = numpy.vstack( (energies,source))
-        if self.SOURCE == 1:
+            # energies = numpy.arange(0,500)
+            # elefactor = numpy.log10(10000.0 / 30.0) / 300.0
+            # energies = 10.0 * 10**(energies * elefactor)
+            # source = numpy.ones(energies.size)
+            # tmp = numpy.vstack( (energies,source))
+            if self.input_spectrum is None:
+                raise Exception("No input beam")
+            else:
+                energies = self.input_spectrum[0,:].copy()
+                source = self.input_spectrum[1,:].copy()
+        elif self.SOURCE == 1:
             energies = numpy.linspace(self.ENER_MIN,self.ENER_MAX,self.ENER_N)
             source = numpy.ones(energies.size)
             tmp = numpy.vstack( (energies,source))
-
-        if self.SOURCE >= 2:
+            self.input_spectrum = source
+        elif self.SOURCE == 2:
             if self.SOURCE == 2: source_file = self.SOURCE_FILE
-            if self.SOURCE == 3: source_file = "SRCOMPE"
-            if self.SOURCE == 4: source_file = "SRCOMPF"
+            # if self.SOURCE == 3: source_file = "SRCOMPE"
+            # if self.SOURCE == 4: source_file = "SRCOMPF"
             try:
                 tmp = numpy.loadtxt(source_file)
                 energies = tmp[:,0]
                 source = tmp[:,1]
+                self.input_spectrum = source
             except:
                 print("Error loading file %s "%(source_file))
                 raise
@@ -812,8 +856,51 @@ class OWxpower(XoppyWidget):
 
 
 if __name__ == "__main__":
+
+
+    from oasys.widgets.exchange import DataExchangeObject
+
+
+
+    input_data_type = "POWER3D"
+
+    if input_data_type == "POWER":
+        # create fake UNDULATOR_FLUX xoppy exchange data
+        e = numpy.linspace(1000.0, 10000.0, 100)
+        source = e/10
+        received_data = DataExchangeObject("XOPPY", "POWER")
+        received_data.add_content("xoppy_data", numpy.vstack((e,e,source)).T)
+        received_data.add_content("xoppy_code", "US")
+
+    elif input_data_type == "POWER3D":
+        # create unulator_radiation xoppy exchange data
+        from orangecontrib.xoppy.util.xoppy_undulators import xoppy_calc_undulator_radiation
+
+        e, h, v, p, code = xoppy_calc_undulator_radiation(ELECTRONENERGY=6.04,ELECTRONENERGYSPREAD=0.001,ELECTRONCURRENT=0.2,\
+                                           ELECTRONBEAMSIZEH=0.000395,ELECTRONBEAMSIZEV=9.9e-06,\
+                                           ELECTRONBEAMDIVERGENCEH=1.05e-05,ELECTRONBEAMDIVERGENCEV=3.9e-06,\
+                                           PERIODID=0.018,NPERIODS=222,KV=1.68,DISTANCE=30.0,
+                                           SETRESONANCE=0,HARMONICNUMBER=1,
+                                           GAPH=0.001,GAPV=0.001,\
+                                           HSLITPOINTS=41,VSLITPOINTS=41,METHOD=0,
+                                           PHOTONENERGYMIN=7000,PHOTONENERGYMAX=8100,PHOTONENERGYPOINTS=20,
+                                           USEEMITTANCES=1)
+        received_data = DataExchangeObject("XOPPY", "POWER3D")
+        received_data = DataExchangeObject("XOPPY", "UNDULATOR_RADIATION")
+        received_data.add_content("xoppy_data", [p, e, h, v])
+        received_data.add_content("xoppy_code", code)
+
+
+
+
     app = QApplication(sys.argv)
     w = OWxpower()
+
+
+    w.acceptExchangeData(received_data)
+
     w.show()
+
+
     app.exec()
     w.saveSettings()
